/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Microsummary Manager.
 *
 * The Initial Developer of the Original Code is
 * Ryan Flint <rflint@dslr.net>.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

function MicrosummaryManager() {};

MicrosummaryManager.prototype = {

  __dirs: null,
  get _dirs() {
    if (!this.__dirs)
      this.__dirs = Cc["@mozilla.org/file/directory_service;1"].
                    getService(Ci.nsIProperties);
    return this.__dirs;
  },

  __obs: null,
  get _obs() {
    if (!this.__obs)
      this.__obs = Cc["@mozilla.org/observer-service;1"].
                   getService(Ci.nsIObserverService);
      return this.__obs;
  },

  __mss: null,
  get _mss() {
    if (!this.__mss)
      this.__mss = Cc["@mozilla.org/microsummary/service;1"].
                   createInstance(Ci.nsIMicrosummaryService);
    return this.__mss;
  },

  _init: function MM__init() {
    var stor = Cc["@mozilla.org/storage/service;1"].
               getService(Ci.mozIStorageService);
    var file = Cc["@mozilla.org/file/directory_service;1"].
               getService(Ci.nsIProperties).
               get("ProfD", Ci.nsIFile);
    file.append("usummgr.sqlite");

    this._db = stor.openDatabase(file);
    this._db.executeSimpleSQL("CREATE TABLE IF NOT EXISTS data (urn TEXT UNIQUE, filename TEXT, name TEXT, favicon TEXT, lastmod TEXT, author TEXT, url TEXT, description TEXT, usage TEXT)");
    this._obs.addObserver(this, "microsummary-generator-installed", false);
    this._obs.addObserver(this, "microsummary-generator-updated", false);

    this._loadGenerators();
  },

  getGenerators: function MM_getGenerators() {
    var generators = [];
    var stmt = this._db.createStatement("SELECT * FROM data ORDER BY UPPER(name) ASC");
    while (stmt.executeStep()) {
      var generator = { urn        : stmt.getUTF8String(0),
                        filename   : stmt.getUTF8String(1),
                        name       : stmt.getUTF8String(2),
                        favicon    : stmt.getUTF8String(3),
                        description: stmt.getUTF8String(7),
                        QueryInterface: function(aIID) {
                          if (!aIID.equals(Ci.mozIGeneratorDescription) &&
                            !aIID.equals(Ci.nsISupports))
                            throw Cr.NS_ERROR_NO_INTERFACE;

                          return this;
                        }
                      };
      generators.push(generator);
    }
    stmt.reset();
    return new SimpleEnum(generators);
  },
  
  getGenerator: function MM_getGenerator(aUrn) {
    var stmt = this._db.createStatement("SELECT FROM data WHERE urn = ?1 LIMIT 1");
    stmt.bindUTF8StringParameter(0, aUrn);
    stmt.executeStep();
    if (stmt.numEntries == 0) {
      stmt.reset();
      return;
    }

    var generator = {
      urn        : stmt.getUTF8String(0),
      filename   : stmt.getUTF8String(1),
      name       : stmt.getUTF8String(2),
      favicon    : stmt.getUTF8String(3),
      author     : stmt.getUTF8String(5),
      url        : stmt.getUTF8String(6),
      description: stmt.getUTF8String(7),
      usage      : stmt.getUTF8String(8),
      QueryInterface: function(aIID) {
        if (!aIID.equals(Ci.mozIGeneratorDescription) &&
          !aIID.equals(Ci.nsISupports))
          throw Cr.NS_ERROR_NO_INTERFACE;

        return this;
      }
    };
    stmt.reset();
    return generator;
  },

  installGenerator: function MM_installGenerator(aURI) {
    this._mss.addGenerator(aURI);
    //Re-cache the generator directories to find the new one
    this._loadGenerators();
    this._obs.notifyObservers(null, 'redraw-required', null);
  },
  
  uninstallGenerator: function MM_uninstallGenerator(aUrn, aFileName) {
    //Remove this generator from any bookmarks still using it. Otherwise it'll be recreated.
    var bookmarks = this._mss.getBookmarks();
    while (bookmarks.hasMoreElements()) {
      var bm = bookmarks.getNext();
      var msum = this._mss.getMicrosummary(bm);
      
      if (aUrn == msum.generator.uri.spec)
        this._mss.removeMicrosummary(bm);
    }

    //Find the generator using our psuedo-relative file path
    //XXX Expand to custom directories?
    var path = aFileName.split('/');
    var file = null;

    LOG(path[0]);
    if (path[0] == "[profile]")
      file = this._dirs.get('UsrMicsumGens', Ci.nsIFile);
      
    else if (path[0] == "[app]")
      file = this._dirs.get('MicsumGens', Ci.nsIFile);
 
    if (file) {
      for (var i = 1; i < path.length; i++)
        file.append(path[i]);

      LOG(file.path);
      if (file.exists()) {
        file.remove(false);

        this._loadGenerators();
        this._obs.notifyObservers(null, 'redraw-required', null);
      }
    }
    else LOG("no file?");
  },
  
  _loadGenerators: function MM__loadGenerators() {
    //Prepare an array to determine if a generator's already been cached or not
    var gens = [];
    var remgens = [];
    var stmt = this._db.createStatement("SELECT * FROM data");
    while (stmt.executeStep()) {
      gens[stmt.getUTF8String(1)] = stmt.getUTF8String(4);
    }

    //Load from both app and profile specific locations
    var appDir = this._dirs.get('MicsumGens', Ci.nsIFile);
    if (appDir.exists()) {
      var genSet = this._loadGenDir(appDir, gens, '[app]');
      for (var i in genSet) {
        var rgx = /^\[app\]/;
        if (genSet[i] != 0 && rgx.test(i.toString()))
          remgens.push(i), LOG('rem: '+i);
      }  
    }

    var profDir = this._dirs.get('UsrMicsumGens', Ci.nsIFile);
    if (profDir.exists()) {
      var genSet = this._loadGenDir(profDir, gens, '[profile]');
      for (var i in genSet) {
        var rgx = /^\[profile\]/;
        if (genSet[i] != 0 && rgx.test(i.toString()))
          remgens.push(i), LOG('rem: '+i);
      }
    }

    if (remgens.length > 0)
      this._removeMissingGenerators(remgens);
  },

  _loadGenDir: function MM__loadGenDir(aDir, aGenSet, aLoc, aApp) {
    var files = aDir.directoryEntries 
                    .QueryInterface(Ci.nsIDirectoryEnumerator);
    var file = files.nextFile;

    while (file) {
      var loc = aLoc + '/' + file.leafName;
      if (file.isDirectory())
        this._loadGenDir(file, aGenSet, loc, aApp);
      else if (!aGenSet[loc])
        this._cacheGenerator(file, false, loc);
      else if (file.lastModifiedTime != aGenSet[loc])
        this._cacheGenerator(file, true, loc), aGenSet[loc] = 0;
      else
        aGenSet[loc] = 0;

      file = files.nextFile;
    }
    files.close();
    return aGenSet;
  },
  
  _removeMissingGenerators: function MM__removeMissingGenerators(aGenSet) {
    for (var i in aGenSet) {
      var stmt = this._db.createStatement("DELETE FROM data WHERE filename = ?1");
      stmt.bindStringParameter(0, aGenSet[i]);
      stmt.execute();
      stmt.reset();
    }
  },
  
  _repairURN: function MM__repairURN(aFile, aXML) {
    var uuidGenerator = Cc["@mozilla.org/uuid-generator;1"].
                        getService(Ci.nsIUUIDGenerator);
    var uuid = uuidGenerator.generateUUID().toString();
    var urn = 'urn:source:' + uuid;

    aXML.documentElement.setAttribute('uri', urn);

    var outputStream = Cc["@mozilla.org/network/safe-file-output-stream;1"].
                       createInstance(Ci.nsIFileOutputStream);
    var localFile = aFile.QueryInterface(Ci.nsILocalFile);
    outputStream.init(localFile, (0x02 | 0x20), 0644, 0);
    var serializer = Cc["@mozilla.org/xmlextras/xmlserializer;1"].
                     createInstance(Ci.nsIDOMSerializer);
    serializer.serializeToStream(aXML, outputStream, null);
    if (outputStream instanceof Ci.nsISafeOutputStream) {
      try       { outputStream.finish() }
      catch (e) { outputStream.close()  }
    }
    else
      outputStream.close();
  },
  
  _cacheGenerator: function MM__cacheGenerator(aFile, aUpdate, aLoc) {
    var ios = Cc["@mozilla.org/network/io-service;1"].
              getService(Components.interfaces.nsIIOService);
    var fileHandler = ios.getProtocolHandler("file")
                         .QueryInterface(Ci.nsIFileProtocolHandler);
    var url = fileHandler.getURLSpecFromFile(aFile);
    
    var req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
              createInstance(Ci.nsIXMLHttpRequest);
    req.open("GET", url, false);
    req.send(null);

    var dom = req.responseXML;
    var urn = dom.documentElement.getAttribute('uri');
    if (!urn && dom.documentElement.tagName != 'parsererror') {
      this._repairURN(aFile, dom);
      return;
    }

    var name = dom.documentElement.getAttribute('name');
    var inc = dom.getElementsByTagName('include')[0].textContent;
    var com = dom.childNodes[0];
    var mod = aFile.lastModifiedTime;
    if (com.nodeName == '#comment') {
      com = com.textContent;
      LOG(name);
      var author = com.match(/@?[aA]uthor\s*:?\s*(.+)/);
      var uri = com.match(/@?[uU]rl\s*:?\s*(.+)/);
      var usage = com.match(/@?[uU]sage\s*:?\s*(.+)/);
      var desc = com.match(/@?[dD]escription\s*:?\s*(.+)/);

      if (author)
        LOG(author[1]);
      if (uri)
        LOG(uri[1]);
      if (usage)
        LOG(usage[1]);
      if (desc)
        LOG(desc[1]);
    }

    var fav = this._generateFaviconPath(inc);

    try {
      //urn TEXT UNIQUE, filename TEXT, name TEXT, favicon TEXT, lastmod TEXT, author TEXT, url TEXT, description TEXT, usage TEXT 
      var query = "INSERT INTO data VALUES (?1, ?2, ?3, ?4, ?5, ?6, ?7, ?8, ?9)";
      if (aUpdate)
        query = "UPDATE data SET name = ?3, favicon = ?4, lastmod = ?5, author = ?6, url = ?7, description = ?8, usage = ?9 WHERE urn = ?1";

      var stmt = this._db.createStatement(query);
      stmt.bindUTF8StringParameter(0, urn);
      if (!aUpdate)
        stmt.bindUTF8StringParameter(1, aLoc);
      stmt.bindUTF8StringParameter(2, name);
      stmt.bindUTF8StringParameter(3, fav);
      stmt.bindStringParameter(4, mod);
      stmt.bindUTF8StringParameter(5, (author) ? author[1] : "");
      stmt.bindUTF8StringParameter(6, (uri) ? uri[1] : "");
      stmt.bindUTF8StringParameter(7, (desc) ? desc[1] : "");
      stmt.bindUTF8StringParameter(8, (usage) ? usage[1] : "");
      stmt.execute();
      stmt.reset();
    }
    catch (e) {
      LOG(e + " ::: " + this._db.lastErrorString);
    }
  },
  
  _generateFaviconPath: function MM__generateFaviconPath(aPattern) {
    //First check if we should even bother with cleanup
    if (aPattern.match(/https?/)) {
      var one = aPattern.replace(/\((\w+)\|[\w|\|]+\)/g, '$1');
      var foo = one.replace(/[^A-Za-z0-9_\/:\.\*]/g, '');
      var url = foo.replace(/\.\*/g, '');
      var ios = Cc["@mozilla.org/network/io-service;1"].
                getService(Ci.nsIIOService);
      var uri = ios.newURI(url, null, null);

      var fav = uri.prePath;
      if (fav.match(/mozilla\.org/))
        fav = 'http://www.mozilla.org';

      return fav + '/favicon.ico';
    }
  },

  observe: function(aSubject, aTopic, aData) {
    //This notification is only sent on 1.9+
    switch(aTopic) {
      case "microsummary-generator-updated":
      case "microsummary-generator-installed":
        this._loadGenerators();
        this._obs.notifyObservers(null, 'redraw-required', null);
        break;
    }
  }
};

function SimpleEnum(aItems) {
  this._items = aItems;
  this._curIndex = 0;
}

SimpleEnum.prototype = {
  hasMoreElements: function() {
    return this._curIndex < this._items.length;
  },
  
  getNext: function() {
    return this._items[this._curIndex++];
  },
  
  QueryInterface: function(aIID) {
    if (!aIID.equals(Ci.nsISimpleEnumerator) && !aIID.equals(Ci.nsISupports))
      throw Cr.NS_ERROR_NO_INTERFACE;

    return this;
  }
};

function LOG(aMsg) {
  var console = Cc['@mozilla.org/consoleservice;1'].getService(Ci.nsIConsoleService);
  /*var ps = Cc['@mozilla.org/preferences-service;1'].getService(Ci.nsIPrefService);
  var branch = ps.getBranch('extensions.uSumManager.');
  branch.QueryInterface(Ci.nsIPrefBranchInternal);
  
  try { 
    var dbg = ps.getBoolPref('debug');
    if (dbg)*/
      console.logStringMessage(aMsg);
    /*ps = null, branch = null;
  } catch(e) {}*/
}

var gModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType) {
    compMgr = aCompMgr.QueryInterface(Ci.nsIComponentRegistrar);

    for (var key in this._objects) {
      var obj = this._objects[key];
      compMgr.registerFactoryLocation(obj.CID, obj.className, obj.contractID,
                                      aFileSpec, aLocation, aType);
    }
  },

  unregisterSelf: function(aCompMgr, aLocation, aType) {},

  getClassObject: function(aCompMgr, aCID, aIID) {
    if (!aIID.equals(Ci.nsIFactory))
      throw Cr.NS_ERROR_NOT_IMPLEMENTED;

    for (var key in this._objects) {
      if (aCID.equals(this._objects[key].CID))
        return this._objects[key].factory;
    }

    throw Cr.NS_ERROR_NO_INTERFACE;
  },

  _objects: {
    service: {
      CID: Components.ID("{479ca4af-4218-4304-8618-1ff1087023f2}"),
      className: "Microsummary Manager",
      contractID: "@screwedbydesign.com/software/usummgr;1",
      factory: MMFactory = {
        createInstance: function(aOuter, aIID) {
          if (aOuter != null)
            throw Cr.NS_ERROR_NO_AGGREGATION;

          var mm = new MicrosummaryManager();
          mm._init();
          return mm;
        }
      }
    },
  },

  canUnload: function(aCompMgr) { return true; }
};

function NSGetModule(aCompMgr, aFileSpec) { return gModule; }
