/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Microsummary Manager.
 *
 * The Initial Developer of the Original Code is
 * Ryan Flint <rflint@dslr.net>.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */
var uSumMgr = {
  init: function() {
    const Cc = Components.classes, Ci = Components.interfaces;   
    this._mm = Cc["@screwedbydesign.com/software/usummgr;1"].
                 getService(Ci.mozIMicrosummaryManager);
    this._obs = Cc["@mozilla.org/observer-service;1"].
                getService(Ci.nsIObserverService);
    this._obs.addObserver(this, "redraw-required", false);

    if (document.documentElement.id == 'extensionsManager') {
      this._strings = document.getElementById('usummgr-strings');

      var rgrp = document.getElementById('viewGroup');
      var oclick = rgrp.getAttribute('onclick');
      rgrp.setAttribute('onclick', 'uSumMgr.viewSelected();' + oclick);
      rgrp.setAttribute('onselect', 'uSumMgr.viewSelected();');

      var prefServ = Cc["@mozilla.org/preferences-service;1"].
                       getService(Ci.nsIPrefService);
      this._prefBranch = prefServ.getBranch("extensions.uSumManager.");

      var ibtn = document.getElementById('installFileButton');
      ibtn.removeAttribute('command');
      ibtn.setAttribute('oncommand', 'uSumMgr.install()');
    
      this.viewSelected();
    }
  },

  destroy: function() {
    this._prefBranch = null;
    this._strings = null;
  },
  
  observe: function(aSubject, aTopic, aData) {
    if (aTopic == "redraw-required" && document.documentElement.id == 'extensionsManager') {
      this.drawn = false;
      this.redraw();
    }
  },

  viewSelected: function(event) {
    var extBoxItems = document.getElementById('extensionsBox').childNodes;
    var microSel = document.getElementById('micro-view').selected;
    var extSel = document.getElementById('extensions-view').selected;
    var themeSel = document.getElementById('themes-view').selected;

    for (var i = 0; i < extBoxItems.length; i++) {
      if ((!themeSel && !microSel) && (extBoxItems[i].id == 'themeSplitter' || extBoxItems[i].id == 'themePreviewArea'))
        continue;

      extBoxItems[i].setAttribute('hidden', (extBoxItems[i].id != 'msumBox') ? microSel : !microSel);
    }

    if (!microSel)
      return;

    gView = 'micro';
    document.getElementById('viewGroup').setAttribute('last-selected', 'micro');
    this.redraw();

    setTimeout("document.getElementById('msumBox').ensureSelectedElementIsVisible()", 1);

    var getMore = document.getElementById('getMore');
    var updates = document.getElementById('checkUpdatesAllButton');
    var install = document.getElementById('installFileButton');
    var restart = document.getElementById('restartAppButton');

    getMore.setAttribute('value', this._strings.getString('usummgr.getMore'));
    getMore.setAttribute('hidden', 'false');
    getMore.setAttribute("getMoreURL", this._prefBranch.getCharPref('getMoreLink'));
    updates.setAttribute('hidden', 'true');
    // Override extensions.hideInstallButton
    install.setAttribute('hidden', 'false');
    restart.setAttribute('hidden', 'true');
  },

  redraw: function() {
    const kXUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

    if (this.drawn)
      return;

    var ubox = document.getElementById('msumBox');
    while (ubox.firstChild) {
      ubox.removeChild(ubox.firstChild);
    }

    var generators = this._mm.getGenerators();

    while (generators.hasMoreElements()) {
      var gen = generators.getNext().QueryInterface(Components.interfaces.mozIGeneratorDescription);
      var ritem = document.createElementNS(kXUL_NS, 'richlistitem');
      var hbox1 = document.createElementNS(kXUL_NS, 'hbox');
      var hbox2 = document.createElementNS(kXUL_NS, 'hbox');
      var vbox1 = document.createElementNS(kXUL_NS, 'vbox');
      var vbox2 = document.createElementNS(kXUL_NS, 'vbox');
      var stack = document.createElementNS(kXUL_NS, 'stack');
      var space = document.createElementNS(kXUL_NS, 'spacer');
      var img = document.createElementNS(kXUL_NS, 'image');
      var lab1 = document.createElementNS(kXUL_NS, 'label');
      var lab2 = document.createElementNS(kXUL_NS, 'label');

      var hbox3 = document.createElementNS(kXUL_NS, 'hbox');
      var space2 = document.createElementNS(kXUL_NS, 'spacer');
      var but1 = document.createElementNS(kXUL_NS, 'button');
      var but2 = document.createElementNS(kXUL_NS, 'button');

      hbox1.setAttribute('flex', '1');
      vbox2.setAttribute('flex', '1');
      hbox3.setAttribute('flex', '1');

      hbox3.id = 'selectedButtons';
      hbox3.setAttribute('style', 'display: none');
      space2.setAttribute('flex', '1');
      but1.setAttribute('label', this._strings.getString('options.button'));
      but1.setAttribute('accesskey', this._strings.getString('options.accesskey'));
      but1.setAttribute('oncommand', 'uSumMgr.options(event)');
      but1.setAttribute('disabled', 'true');
      but2.setAttribute('label', this._strings.getString('uninstall.button'));
      but2.setAttribute('accesskey', this._strings.getString('uninstall.accesskey'));
      but2.setAttribute('oncommand', 'uSumMgr.uninstall(event)');
      hbox3.appendChild(but1);
      hbox3.appendChild(space2);
      hbox3.appendChild(but2);

      ritem.id = gen.urn;
      ritem.setAttribute('fileName', gen.filename);
      ritem.setAttribute('name', gen.name);
      ritem.setAttribute('class', 'msum');

      vbox1.id = 'addon-icon';
      stack.id = 'addonIconStack';
      space.setAttribute('flex', '1');
      img.setAttribute('class', 'msumImg');
      img.setAttribute('src', gen.favicon);
      img.setAttribute('onerror', 'this.removeAttribute("src");');

      stack.appendChild(img);
      vbox1.appendChild(stack);
      vbox1.appendChild(space);
      hbox1.appendChild(vbox1);

      vbox2.id = 'addonTextBox';
      hbox2.id = 'addon-name-version';
      lab1.id = 'addonName';
      lab1.setAttribute('value', gen.name);
      hbox2.appendChild(lab1);
      vbox2.appendChild(hbox2);
      lab2.setAttribute('value', gen.description);
      vbox2.appendChild(lab2);
      vbox2.appendChild(hbox3);
      hbox1.appendChild(vbox2);

      ritem.appendChild(vbox1);
      ritem.appendChild(vbox2);
      ubox.appendChild(ritem);
    }

    this.drawn = true;

    var last = ubox.getAttribute('last-selected');
    if (last)
      var sel = document.getElementById(last);
    else
      var sel = ubox.firstChild;
      
    ubox.selectedItem = sel;
  },
  
  onSelect: function(event) {
    var mbox = document.getElementById('msumBox');
    mbox.ensureSelectedElementIsVisible();
  },

  install: function() {
    if (gView != 'micro')
      installWithFilePicker();
    else {
      const Cc = Components.classes, Ci = Components.interfaces;
      var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
      fp.init(window, this._strings.getString('install.title'), nsIFilePicker.modeOpen);
      fp.appendFilter(this._strings.getString('install.types'), "*.xml");
      fp.appendFilters(nsIFilePicker.filterAll);

      var ret = fp.show();
      if (ret == nsIFilePicker.returnOK) {
        var ioService = Components.classes['@mozilla.org/network/io-service;1'].getService(Ci.nsIIOService);
        var fileProtocolHandler = ioService.getProtocolHandler("file").QueryInterface(Ci.nsIFileProtocolHandler);
        var url = fileProtocolHandler.newFileURI(fp.file).QueryInterface(nsIURL);
        if (url)
          this._mm.installGenerator(url);
      }
    }
  },

  uninstall: function(event) {
    const Cc = Components.classes, Ci = Components.interfaces;
    var ps = Cc["@mozilla.org/embedcomp/prompt-service;1"].
               getService(Components.interfaces.nsIPromptService);
    var elem = document.getElementById('msumBox').selectedItem;
    var name = this._strings.getFormattedString('uninstall.confirm', [elem.getAttribute('name')]);
    var conf = ps.confirm(window, name, name);
    if (!conf)
      return;

    this._mm.uninstallGenerator(elem.id, elem.getAttribute('fileName'));
  },

  options: function(event) {
    window.openDialog("chrome://usummgr/content/uSumDialog.xul","showmore","chrome,modal,centerscreen");
  }
};

var uSumMgrDNDObserver = {
  _ioServ: null,
  _canDrop: false,

  _ensureServices: function() {
    if (!this._ioServ)
      this._ioServ = Components.classes["@mozilla.org/network/io-service;1"]
                               .getService(nsIIOService);
  },

  _getDataFromDragSession: function (aDragSession, aPosition) {
    var fileData = { };
    // if this fails we do not have valid data to drop
    try {
      var xfer = Components.classes["@mozilla.org/widget/transferable;1"]
                           .createInstance(Components.interfaces.nsITransferable);
      xfer.addDataFlavor("text/x-moz-url");
      xfer.addDataFlavor("application/x-moz-file", "nsIFile");
      aDragSession.getData(xfer, aPosition);

      var flavour = { }, data = { }, length = { };
      xfer.getAnyTransferData(flavour, data, length);
      var selectedFlavour = this.getSupportedFlavours().flavourTable[flavour.value];
      var xferData = new FlavourData(data.value, length.value, selectedFlavour);

      var fileURL = transferUtils.retrieveURLFromData(xferData.data,
                                                      xferData.flavour.contentType);
      fileData.fileURL = fileURL;

      var uri = this._ioServ.newURI(fileURL, null, null);
      var url = uri.QueryInterface(nsIURL);
      fileData.fileName = url.fileName;

      switch (url.fileExtension) {
        case "xpi":
          fileData.type = Components.interfaces.nsIUpdateItem.TYPE_EXTENSION;
          break;
        case "jar":
          fileData.type = Components.interfaces.nsIUpdateItem.TYPE_THEME;
          break;
        case "xml":
          fileData.type = 'msum';
          break;
        default:
          return null;
      }
    }
    catch (e) {
      return null;
    }

    return fileData;
  },

  canDrop: function (aEvent, aDragSession) { return this._canDrop; },

  onDragEnter: function (aEvent, aDragSession) {
    this._ensureServices();
    //XXX This may cause crashes on GTK2
    var count = aDragSession.numDropItems;
    for (var i = 0; i < count; ++i) {
      var fileData = this._getDataFromDragSession(aDragSession, i);
      if (!fileData) {
        this._canDrop = false;
        return;
      }
    }
    this._canDrop = true;
  },

  onDragOver: function (aEvent, aFlavor, aDragSession) { },

  onDrop: function(aEvent, aXferData, aDragSession) {
    const Cc = Components.classes, Ci = Components.interfaces;
    if (!isXPInstallEnabled())
      return;

    this._ensureServices();
    
    var xpinstallObj = { };
    var themes = { };
    var microsummaries = { };
    var xpiCount = 0;
    var themeCount = 0;
    var msumCount = 0;

    var count = aDragSession.numDropItems;
    for (var i = 0; i < count; ++i) {
      var fileData = this._getDataFromDragSession(aDragSession, i);
      if (!fileData)
        continue;

      if (fileData.type == nsIUpdateItem.TYPE_EXTENSION) {
        xpinstallObj[fileData.fileName] = fileData.fileURL;
        ++xpiCount;
      }
      else if (fileData.type == nsIUpdateItem.TYPE_THEME) {
        themes[fileData.fileName] = fileData.fileURL;
        ++themeCount;
      }
      else if (fileData.type == 'msum') {
        microsummaries[fileData.fileName] = fileData.fileURL;
        ++msumCount;
      }
    }

    if (xpiCount > 0) 
      InstallTrigger.install(xpinstallObj), showView('installs'), uSumMgr.viewSelected();
    if (themeCount > 0) {
      // XXX bug 257992
      for (var fileName in themes) {
        InstallTrigger.installChrome(InstallTrigger.SKIN, themes[fileName], fileName);
        showView('installs');
        uSumMgr.viewSelected();
        break;
      }
    }
    if (msumCount > 0) {
      for (var fileName in microsummaries) {
        var ioService = Components.classes['@mozilla.org/network/io-service;1'].getService(Ci.nsIIOService);
        var uri = ioService.newURI(microsummaries[fileName], null, null);
        uSumMgr.addMsum(uri);
      }
    }
  },

  _flavourSet: null,  
  getSupportedFlavours: function() {
    if (!this._flavourSet) {
      this._flavourSet = new FlavourSet();
      this._flavourSet.appendFlavour("text/x-moz-url");
      this._flavourSet.appendFlavour("application/x-moz-file", "nsIFile");
    }
    return this._flavourSet;
  }
};

window.addEventListener("load", function() { uSumMgr.init(); }, false);
window.addEventListener("close", function() { uSumMgr.destroy(); }, false);
